@@ -4,11 +4,10 @@ from functools import partial
 from itertools import groupby
 
 from trytond.pool import PoolMeta
+from trytond.tools import sortable_values
 
 from .common import parcel_weight
 
-__all__ = ['Sale']
-
 
 class Sale(metaclass=PoolMeta):
     __name__ = 'sale.sale'
@@ -19,10 +18,14 @@ class Sale(metaclass=PoolMeta):
         """
         return ()
 
+    def _parcel_weight(self, parcel):
+        if self.carrier:
+            return parcel_weight(parcel, self.carrier.weight_uom, 'unit')
+
     def _get_carrier_context(self):
         context = super(Sale, self)._get_carrier_context()
 
-        if self.carrier.carrier_cost_method != 'weight':
+        if not self.carrier or self.carrier.carrier_cost_method != 'weight':
             return context
         context = context.copy()
         weights = []
@@ -30,9 +33,8 @@ class Sale(metaclass=PoolMeta):
 
         lines = [l for l in self.lines or [] if l.quantity and l.quantity > 0]
         keyfunc = partial(self._group_parcel_key, lines)
-        lines = sorted(lines, key=keyfunc)
+        lines = sorted(lines, key=sortable_values(keyfunc))
 
         for key, parcel in groupby(lines, key=keyfunc):
-            weights.append(parcel_weight(
-                    parcel, self.carrier.weight_uom, 'unit'))
+            weights.append(self._parcel_weight(parcel))
         return context
