@@ -9,12 +9,14 @@ from trytond.tests.test_tryton import doctest_teardown
 from trytond.tests.test_tryton import doctest_checker
 from trytond.pool import Pool
 
+from trytond.modules.company.tests import CompanyTestMixin
 from trytond.modules.currency.tests import create_currency
 
 
-class CarrierWeightTestCase(ModuleTestCase):
+class CarrierWeightTestCase(CompanyTestMixin, ModuleTestCase):
     'Test CarrierWeight module'
     module = 'carrier_weight'
+    extras = ['purchase_shipment_cost', 'sale_shipment_cost']
 
     @with_transaction()
     def test_compute_weight_price(self):
